#!/usr/bin/env perl
## 
## Copyright (C) 2011-2018 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 


use strict;
use Getopt::Long;
use List::Util 'shuffle';

my $cvsid = '$Id: find-untested-pairs.pl,v 1.7 2012/06/04 18:51:10 kkeys Exp $';
my $cmdline = join(' ', $0, @ARGV);

my $usage =
    "Usage: $0 [<options>] <pair_file> <res_result_files>...\n" .
    "Options:\n" .
#    "--outdir <dir>    write results to <dir>\n" .
    "--include <file>  include only addresses in <file>\n" .
    "\n";

sub usageExit() {
    print STDERR $usage;
    exit 1;
}

# parameters
my $incfile = undef;

GetOptions(
#    "outdir=s" => \$outdir,
    "include=s" => \$incfile
    ) or usageExit();

usageExit() if @ARGV < 2;

#### state
my %results;
my %include;
my %testedresult;

sub loadResulttypes() {
    my $file = "results.h";
    open RH, "<", $file || die "$file: $!\n";
    while (<RH>) {
	if (/^\s*defresult\(\s*(\w+),\s*(\w+)\s*\)/) {
	    my $label = $1;
	    my $type = $2;
	    if ($type =~ /^(UN)?SHARED$/) {
		$testedresult{$label} = 1;
	    }
	}
    }
    close RH;
}

sub loadInclude($) {
    my ($file) = @_;
    print STDERR "# loading addresses from $file\n";
    open INC, "<", $file || die "$file: $!\n";
    my ($setid, $set, $size, $vlo, $vhi, $good, $large);
    $setid = -1;
    while (<INC>) {
	next if (/^#/ || /^$/);
	chomp;
	$include{$_}++;
    }
    close INC;
}

sub loadResults() {
    while (@ARGV) {
	my $file = $ARGV[0];
	print STDERR "# loading results from $file\n";
	open DATA, "<", $file || die "$file: $!\n";
	while (<DATA>) {
	    next if (/^#/ || /^$/);
	    my ($addr1, $src1, $meth1, $v1, $addr2, $src2, $meth2, $v2, $vratio, $result, $bnd, $int, $ext, $overlap, $oob, $idist, $edist) = split;
	    if (!defined $oob) {
		die "$file:$.: syntax error\n";
	    }
	    if ($testedresult{$result}) {
		$results{"$addr1:$addr2"}++;
	    }
	}
	close DATA;
	shift @ARGV;
    }
}

sub loadPairs($) {
    my ($file) = @_;
    print STDERR "# loading pairs from $file\n";
    open PAIRS, "<", $file || die "$file: $!\n";
    my ($setid, $set, $size, $vlo, $vhi, $good, $large);
    $setid = -1;
    while (<PAIRS>) {
	next if (/^#/ || /^$/);
	my ($addr1, $addr2) = split;
	next if ($incfile && (!$include{$addr1} || !$include{$addr2}));
	if (!$results{"$addr1:$addr2"} && !$results{"$addr2:$addr1"}) {
	    print;
	}
    }
    close PAIRS;
}

my $pairfile = $ARGV[0];
shift @ARGV;
loadResulttypes();
loadInclude($incfile);
loadResults();
loadPairs($pairfile);

